@@ -3,7 +3,7 @@ use 5.008001;
 use strict;
 use warnings;
 
-our $VERSION = '1.049_003';
+our $VERSION = '1.049_004';
 
 =head1 NAME
 
@@ -822,7 +822,8 @@ sub pack_manifest_hash {
     # generate a selective set of filters from the options passed in via -F
     my $mod_filter = _generate_filter($opt, 'F');
 
-    my ($privlib, $archlib) = map { _maybe_relocate($_) } @Config{qw(privlib archlib)};
+    my ($privlib, $archlib) = map { (my $lib = $_) =~ s{\\}{/}g; $lib } 
+                                  @Config{qw(privlibexp archlibexp)};
 
     foreach my $pfile (sort grep length $map{$_}, keys %map) {
         next if !$opt->{B} and (
@@ -939,18 +940,6 @@ sub pack_manifest_hash {
 }
 
 
-sub _maybe_relocate {
-    my ($lib) = @_;
-
-    if ($Config{userelocatableinc})
-    {
-        $lib =~ s/^\.\.\./dirname($^X)/e;
-        $lib = abs_path($lib);
-    }
-    $lib =~ s{\\}{/}g;
-    return $lib;
-}
-
 sub _generate_filter {
     my $opt = shift; # options hash
     my $key = shift; # F or f? modules or script?
