@@ -2,7 +2,6 @@ XS(XS_Internals_PAR_BOOT) {
     GV* tmpgv;
     AV* tmpav;
     SV** svp;
-    SV* tmpsv;
     int i;
     int ok = 0;
     char *buf;
@@ -24,7 +23,7 @@ XS(XS_Internals_PAR_BOOT) {
         }
         if (ok == options_count - 1) {
             for (i = 1; i < options_count; i++) {
-                tmpsv = av_shift(tmpav);
+                SV* unused = av_shift(tmpav);
             }
         }
     }
@@ -87,7 +86,7 @@ XS(XS_Internals_PAR_BOOT) {
      * this scalar so that a packed script may refer to the version
      * of PAR::Packer it was built with.
      */
-    sv_setpv(get_sv("PAR::Packer::VERSION", GV_ADD), PP_VERSION);
+    sv_setpv(get_sv("PAR::Packer::VERSION", GV_ADD), PAR_PACKER_VERSION);
 
     TAINT_NOT;
 
